program ClistExample;

uses glib2, gdk2, gtk2;

{ User clicked the "Add List" button. }

procedure button_add_clicked (data: PGtkCList); cdecl;
const
   { Something silly to add to the list. 4 rows of 2 columns each }
   drink : array [0..3, 0..1] of pgchar = 
   (('Milk', '3 Oz'),		      
    ('Water', '6 l'),
    ('Carrots', '2'),
    ('Snakes', '55'));
var
   indx : integer;
begin
   { Here we do the actual adding of the text. It's done once for each row. }
   for indx := 0 to 3 do
      gtk_clist_append(data, @drink[indx]);
end;

{ User clicked the "Clear List" button. }

procedure button_clear_clicked (data : PGtkCList); cdecl;
begin
   { Clear the list using gtk_clist_clear. This is much faster than calling
     gtk_clist_remove once for each row. }
   gtk_clist_clear(data);
end;

{ The user clicked the "Hide/Show titles" button. }

var
   { Just a flag to remember the status. False = currently visible }
   hide_show_flag : boolean = false;
   
procedure button_hide_show_clicked (data : PGtkCList); cdecl;
begin
   if not hide_show_flag then begin
      { Hide the titles and set the flag to true }
      gtk_clist_column_titles_hide(data);
      hide_show_flag := true;
   end
   else begin
      { Show the titles and reset flag to false }
      gtk_clist_column_titles_show(data);
      hide_show_flag := false;
   end;
end;

{ If we come here, then the user has selected a row in the list. }

procedure selection_made (clist : PGtkCList;
                          row, column: gint;
                          event : PGdkEventButton;
                          data : gpointer); cdecl;
var
   text : pgchar;
begin
   { Get the text that is stored in the selected row and column which was
   clicked in.  We will receive it as a pointer in the argument text. }
   gtk_clist_get_text(clist, row, column, @text);

   { Just prints some information about the selected row }
   writeln('You selected row ', row,
           '. More specifically you clicked in column ', column,
           ', and the text in this cell is ', text);
end;

const
   titles : array [0..1] of pgchar = ('Ingredients', 'Amount');

var
   window, vbox, hbox, scrolled_window, clist : PGtkWidget;
   button_add, button_clear, button_hide_show : PGtkWidget;

begin
   gtk_init(@argc, @argv);

   window := gtk_window_new(gtk_WINDOW_TOPLEVEL);
   gtk_widget_set_size_request(PGtkWIDGET(window), 300, 150);

   gtk_window_set_title(GTK_WINDOW(window), 'GtkCList Example');
   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@gtk_main_quit), nil);

   vbox := gtk_vbox_new(false, 5);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
   gtk_container_add(GTK_CONTAINER(window), vbox);
   gtk_widget_show(vbox);

   { Create the ScrolledWindow to pack the CList widget into it. }
   scrolled_window := gtk_scrolled_window_new(nil, nil);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

   gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, true, true, 0);
   gtk_widget_show(scrolled_window);

   { Create the CList. For this example we use 2 columns }
   clist := gtk_clist_new_with_titles(2, titles);

   { When a selection is made, we want to know about it. The callback
   used is selection_made, and it's code can be found above }
   g_signal_connect(clist, 'select_row',
                    G_CALLBACK(@selection_made), nil);

   { It isn't necessary to shadow the border, but it looks nice :) }
   gtk_clist_set_shadow_type(GTK_CLIST(clist), GTK_SHADOW_OUT);

   { What however is important, is that we set the column widths as
    they will never be right otherwise. Note that the columns are
    numbered from 0 and up (to 1 in this case). }
   gtk_clist_set_column_width(GTK_CLIST(clist), 0, 150);
   gtk_clist_set_column_width(GTK_CLIST(clist), 1, 100);

   { Add the CList widget to the vertical box and show it. }
   gtk_container_add(GTK_CONTAINER(scrolled_window), clist);
   gtk_widget_show(clist);

   { Create the buttons and add them to the window. See the button
    tutorial for more examples and comments on this. }
   hbox := gtk_hbox_new(false, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, false, true, 0);
   gtk_widget_show(hbox);

   button_add := gtk_button_new_with_label('Add List');
   button_clear := gtk_button_new_with_label('Clear List');
   button_hide_show := gtk_button_new_with_label('Hide/Show titles');

   gtk_box_pack_start(GTK_BOX(hbox), button_add, true, true, 0);
   gtk_box_pack_start(GTK_BOX(hbox), button_clear, true, true, 0);
   gtk_box_pack_start(GTK_BOX(hbox), button_hide_show, true, true, 0);

   { Connect our callbacks to the three buttons }
   g_signal_connect_swapped(button_add, 'clicked',
			    G_CALLBACK(@button_add_clicked), clist);
   g_signal_connect_swapped(button_clear, 'clicked',
			    G_CALLBACK(@button_clear_clicked), clist);
   g_signal_connect_swapped(button_hide_show, 'clicked',
			    G_CALLBACK(@button_hide_show_clicked), clist);

   gtk_widget_show(button_add);
   gtk_widget_show(button_clear);
   gtk_widget_show(button_hide_show);

   { The interface is completely set up so we show all the widgets and
   enter the gtk_main loop }
   gtk_widget_show(window);

   gtk_main();
end.

{*EOF*}
