program Colorsel;

uses glib2, gdk2, gtk2;

var
   colorseldlg : PGtkWidget = nil;
   drawingarea : PGtkWidget = nil;
   color : TGdkColor;

{ Color changed handler }

procedure color_changed_cb (Widget : PGtkWidget;
                            colorsel : PGtkColorSelection); cdecl;
var
   ncolor : TGdkColor;
begin
   gtk_color_selection_get_current_color(colorsel, @ncolor);
   gtk_widget_modify_bg(drawingarea, GTK_STATE_NORMAL, @ncolor);       
end;

{ Drawingarea event handler }

function area_event (widget : PGtkWidget; event : pGdkEvent;
                     client_data : gpointer) : boolean; cdecl;
var
   handled  : boolean = false;
   response : integer;
   colorsel : PGtkColorSelection;

begin
   { Check if we've received a button pressed event }
   if (event^._type = GDK_BUTTON_PRESS) then
   begin
      handled := true;

      { Create color selection dialog }
      if (colorseldlg = nil) then
         colorseldlg := gtk_color_selection_dialog_new('Select background color');

      { Get the ColorSelection widget }
      colorsel := GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colorseldlg)^.colorsel);

      gtk_color_selection_set_previous_color(colorsel, @color);
      gtk_color_selection_set_current_color(colorsel, @color);
      gtk_color_selection_set_has_palette(colorsel, true);

      { Connect to the "color_changed" signal, set the client-data
      to the colorsel widget }
      g_signal_connect(colorsel, 'color_changed',
                       G_CALLBACK(@color_changed_cb), gpointer(colorsel));

      { Show the dialog }
      response := gtk_dialog_run(GTK_DIALOG(colorseldlg));
      
      if (response = GTK_RESPONSE_OK) then
         gtk_color_selection_get_current_color(colorsel, @color)
      else 
         gtk_widget_modify_bg(drawingarea, GTK_STATE_NORMAL, @color);

      gtk_widget_hide(colorseldlg);
   end;
   area_event := handled;
end;

{Close down and exit handler}

function destroy_window (widget : PGtkWidget; event : PGdkEvent;
                         data   : gpointer) : boolean; cdecl;
begin
   gtk_main_quit();
   destroy_window := true;
end;

var
   window : PGtkWidget;

begin
   { Initialize the toolkit, remove gtk-related commandline stuff }
   gtk_init(@argc, @argv);
   
   { Create toplevel window, set title and policies. }
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(window), 'Color selection test');
   gtk_window_set_resizable(GTK_WINDOW(window), true);

   { Attach to the "delete" and "destroy" events so we can exit }
   g_signal_connect(window, 'delete-event',
                    G_CALLBACK(@destroy_window), gpointer(window));

   { Create drawingarea, set size and catch button events }
   drawingarea := gtk_drawing_area_new();

   color.red := 0;
   color.blue := 65535;
   color.green := 0;
   gtk_widget_modify_bg(drawingarea, GTK_STATE_NORMAL, @color);       
   
   gtk_widget_set_size_request(GTK_WIDGET(drawingarea), 200, 200);

   gtk_widget_set_events(drawingarea, GDK_BUTTON_PRESS_MASK);

   g_signal_connect(drawingarea, 'event',
                    G_CALLBACK(@area_event), gpointer(drawingarea));

   { Add drawingarea to window, then show them both }
   gtk_container_add(GTK_CONTAINER(window), drawingarea);

   gtk_widget_show(drawingarea);
   gtk_widget_show(window);

   { Enter the gtk_main loop (this never returns) }
   gtk_main();
end.

{*EOF*}
