program Gettargets;

uses glib2, gtk2, gdk2;

{ Signal handler invoked when user clicks on the "Get Targets" button }

var
   targets_atom : TGdkAtom;

procedure get_targets (widget : PGtkWidget; data : gpointer); cdecl;
begin
   { Get the atom corresponding to the string "TARGETS" }
   if targets_atom = GDK_NONE then
      targets_atom := gdk_atom_intern('TARGETS', false);
   
   { And request the "TARGETS" target for the primary selection }
   gtk_selection_convert(GTK_WIDGET(data), GDK_SELECTION_PRIMARY, targets_atom,
                         GDK_CURRENT_TIME);
end;

{ Signal handler called when the selections owner returns the data }

procedure selection_received (widget : PGtkWidget;
			      selection_data : PGtkSelectionData;
                              data : gpointer); cdecl;
var
   atoms	   : PGdkAtom;
   i, num_elements : Integer;
   name		   : pchar;

begin
   { **** IMPORTANT **** Check to see if retrieval succeeded }
   if (selection_data^.length < 0) then
      writeln('Selection retrieval failed')
   { Make sure we got the data in the expected form }
   else if selection_data^._type <> GDK_SELECTION_TYPE_ATOM then
      writeln('Selection "TARGETS" was not returned as atoms!')
   else begin
      { Print out the atoms we received }
      atoms := pGdkAtom(selection_data^.data);

      num_elements := trunc((selection_data^.length) / sizeof(TGdkAtom));
      for i := 0 to num_elements - 1 do begin
         name := gdk_atom_name(atoms[i]);
         if name <> nil then
	    writeln(name)
	 else
	    writeln('(bad atom)');
      end;
   end;
end;

var
   window, button : PGtkWidget;

begin
   gtk_init(@argc, @argv);
   targets_atom := GDK_NONE;
   
   { Create the toplevel window }
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(window), 'Event Box');
   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@gtk_main_quit), nil);

   { Create a button the user can click to get targets }
   button := gtk_button_new_with_label('Get Targets');
   gtk_container_add(GTK_CONTAINER(window), button);
   
   g_signal_connect(button, 'clicked',
                    G_CALLBACK(@get_targets), window);
   g_signal_connect(window, 'selection_received',
                    G_CALLBACK(@selection_received), nil);

   gtk_widget_show(button);
   gtk_widget_show(window);
   
   gtk_main();
end.

{*EOF*}
