unit gtkdial;

{ GTK - The GIMP Toolkit
  Copyright for C version (C) 1995-1997 Peter Mattis, Spencer Kimball
                                        and Josh MacDonald
  Copyright for Pascal version 2014 Zbigniew Jurkiewicz
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA. }

interface

uses glib2, gdk2, gtk2, math;

type
   PGtkDial = ^TGtkDial;
   TGtkDial = 
      record   
         widget : TGtkWidget;

         { Update policy (GTK_UPDATE_[CONTINUOUS/DELAYED/DISCONTINUOUS]) }
         policy : guint;  {: 2}

         { Button currently pressed or 0 if none }
         button :guint8;

         { Dimensions of dial components }
         radius : gint;
         pointer_width : gint;

         { ID of update timer, or 0 if none }
         timer : guint32;

         { Current angle }
         angle : gfloat;
         last_angle : gfloat;

         { Old values from adjustment stored so we know when something changes }
         old_value : gfloat;
         old_lower : gfloat;
         old_upper : gfloat;

         { The adjustment object that stores the data for this dial }
         adjustment : PGtkAdjustment;
      end;

   PGtkDialClass = ^TGtkDialClass;
   TGtkDialClass = record
                      parent_class : TGtkWidgetClass;
                   end;

function gtk_dial_new (adjustment : PGtkAdjustment) : PGtkWidget;
function gtk_dial_get_type () : GType;
function gtk_dial_get_adjustment (dial : PGtkDial) : PGtkAdjustment;
procedure gtk_dial_set_update_policy (dial : PGtkDial; policy : TGtkUpdateType);

procedure gtk_dial_set_adjustment (dial : PGtkDial;
                                   adjustment : PGtkAdjustment);

function GTK_DIAL (obj : pointer) : PGtkDial;
function GTK_DIAL_CLASS (klass : pointer ) : PGtkDialClass;
function GTK_IS_DIAL (obj : pointer) : boolean;

implementation

function GTK_DIAL (obj : pointer) : PGtkDial;
begin
   GTK_DIAL := PGtkDial(G_TYPE_CHECK_INSTANCE_CAST(obj, gtk_dial_get_type()));
end;

function GTK_DIAL_CLASS (klass : pointer) : PGtkDialClass;
begin
   GTK_DIAL_CLASS :=
      PGtkDialClass(G_TYPE_CHECK_CLASS_CAST(klass, gtk_dial_get_type()));
end;

function GTK_IS_DIAL (obj : pointer) : boolean;
begin
   GTK_IS_DIAL := G_TYPE_CHECK_INSTANCE_TYPE(obj, gtk_dial_get_type());
end;

const
   SCROLL_DELAY_LENGTH = 300; 
   DIAL_DEFAULT_SIZE = 100;

{ Forward declarations }

procedure gtk_dial_class_init (klass : PGtkDialClass) cdecl; forward;
procedure gtk_dial_init (dial : PGtkDial) cdecl; forward;
procedure gtk_dial_destroy (_object : PGtkObject); cdecl; forward;
procedure gtk_dial_realize (widget : PGtkWidget); cdecl; forward;
procedure gtk_dial_size_request (widget : PGtkWidget;
                                 requisition : PGtkRequisition); cdecl; forward;
procedure gtk_dial_size_allocate (widget : PGtkWidget;
                                  allocation : PGtkAllocation); cdecl; forward;
function gtk_dial_expose (widget : PGtkWidget; event : PGdkEventExpose) : gboolean; cdecl; forward;
function gtk_dial_button_press (widget : PGtkWidget;
                                event : PGdkEventButton) : gboolean; cdecl; forward;
function gtk_dial_button_release (widget : PGtkWidget;
                                  event  : PGdkEventButton): gboolean; cdecl; forward;
function gtk_dial_motion_notify (widget : PGtkWidget;
                                 event : PGdkEventMotion) : gboolean; cdecl; forward;
function gtk_dial_timer (data : gpointer) : gboolean; cdecl; forward;

procedure gtk_dial_update_mouse (dial : PGtkDial; x : gint; y : gint); forward;
procedure gtk_dial_update (dial : PGtkDial); forward;
procedure gtk_dial_adjustment_changed (adjustment : PGtkAdjustment;
                                       data : gpointer); cdecl; forward;
procedure gtk_dial_adjustment_value_changed (adjustment : PGtkAdjustment;
                                             data : gpointer); cdecl; forward;

{ Local data }

var
   parent_class : PGtkWidgetClass = nil; 

var
   dial_type : GType = 0; 

function gtk_dial_get_type () : GType;
const
   dial_info: TGTypeInfo = 
   (class_size: sizeof(TGtkDialClass);
    base_init: nil;    
    base_finalize: nil;
    class_init: TGClassInitFunc(@gtk_dial_class_init);
    class_finalize: nil;
    class_data: nil;   
    instance_size: sizeof(TGtkDial);
    n_preallocs: 0;
    instance_init: TGInstanceInitFunc(@gtk_dial_init););
begin
   if dial_type = 0 then
      dial_type := g_type_register_static(GTK_TYPE_WIDGET, 'GtkDial', 
                                          @dial_info, 0);

   gtk_dial_get_type := dial_type;
end;

procedure gtk_dial_class_init (klass : PGtkDialClass); cdecl;
var
   object_class : PGtkObjectClass;
   widget_class : PGtkWidgetClass;
begin
   object_class := PGtkObjectClass(klass);
   widget_class := PGtkWidgetClass(klass);

   parent_class := g_type_class_peek_parent(klass);

   object_class^.destroy := @gtk_dial_destroy;

   widget_class^.realize := @gtk_dial_realize;
   widget_class^.expose_event := @gtk_dial_expose;
   widget_class^.size_request := @gtk_dial_size_request;
   widget_class^.size_allocate := @gtk_dial_size_allocate;
   widget_class^.button_press_event := @gtk_dial_button_press;
   widget_class^.button_release_event := @gtk_dial_button_release;
   widget_class^.motion_notify_event := @gtk_dial_motion_notify;
end;

procedure gtk_dial_init (dial : PGtkDial); cdecl;
begin
   dial^.button := 0;
   dial^.policy := GTK_UPDATE_CONTINUOUS;
   dial^.timer := 0;
   dial^.radius := 0;
   dial^.pointer_width := 0;
   dial^.angle := 0.0;
   dial^.old_value := 0.0;
   dial^.old_lower := 0.0;
   dial^.old_upper := 0.0;
   dial^.adjustment := nil;
end;

function gtk_dial_new (adjustment : PGtkAdjustment) : PGtkWidget;
var
   dial : PGtkDial; 
begin
   dial := g_object_new(gtk_dial_get_type(), nil);

   if adjustment = nil then
      adjustment := PGtkAdjustment(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0,
                                                      0.0, 0.0));

   gtk_dial_set_adjustment(dial, adjustment);
   
   gtk_dial_new := GTK_WIDGET(dial);
end;

procedure gtk_dial_destroy (_object : PGtkObject); cdecl;
var
   dial : PGtkDial;
begin
   if (_object = nil) or not GTK_IS_DIAL(_object) then
      Exit();

   dial := GTK_DIAL(_object);

   if dial^.adjustment <> nil then begin
      g_object_unref(GTK_OBJECT(dial^.adjustment));
      dial^.adjustment := nil;
   end;

   {GTK_OBJECT_CLASS(parent_class)^.destroy(object);}
end;

function gtk_dial_get_adjustment (dial : PGtkDial) : PGtkAdjustment;
begin
   if (dial <> nil) and GTK_IS_DIAL(dial) then
      gtk_dial_get_adjustment := dial^.adjustment
   else
      gtk_dial_get_adjustment := nil;
end;

procedure gtk_dial_set_update_policy (dial : PGtkDial; policy : TGtkUpdateType);
begin
   if (dial <> nil) and GTK_IS_DIAL(dial) then
      dial^.policy := policy;
end;

procedure gtk_dial_set_adjustment (dial : PGtkDial;
                                   adjustment : PGtkAdjustment);
begin
   if (dial = nil) or not GTK_IS_DIAL(dial) then
      Exit();

   if dial^.adjustment <> nil then begin
      g_signal_handlers_disconnect_by_func(GTK_OBJECT(dial^.adjustment), nil,
                                           gpointer(dial));
      g_object_unref(GTK_OBJECT(dial^.adjustment));
   end;

   dial^.adjustment := adjustment;
   g_object_ref(GTK_OBJECT(dial^.adjustment));

   g_signal_connect(G_OBJECT(adjustment), 'changed',
                    G_CALLBACK(@gtk_dial_adjustment_changed),
                    gpointer(dial));
   g_signal_connect(G_OBJECT(adjustment), 'value_changed',
                    G_CALLBACK(@gtk_dial_adjustment_value_changed),
                    gpointer(dial));

   dial^.old_value := adjustment^.value;
   dial^.old_lower := adjustment^.lower;
   dial^.old_upper := adjustment^.upper;

   gtk_dial_update(dial);
end;

procedure gtk_dial_realize (widget : PGtkWidget); cdecl;
var
   dial : PGtkDial; 
   attributes : TGdkWindowAttr;
   attributes_mask : gint;
begin
   if (widget = nil) or not GTK_IS_DIAL(widget) then
      Exit();

   {gtk_widget_set_realized(widget, true);}
   GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);
   dial := GTK_DIAL(widget);

   attributes.x := widget^.allocation.x;
   attributes.y := widget^.allocation.y;
   attributes.width := widget^.allocation.width;
   attributes.height := widget^.allocation.height;
   attributes.wclass := GDK_INPUT_OUTPUT;
   attributes.window_type := GDK_WINDOW_CHILD;
   attributes.event_mask := gtk_widget_get_events(widget) or 
      GDK_EXPOSURE_MASK or GDK_BUTTON_PRESS_MASK or 
      GDK_BUTTON_RELEASE_MASK or GDK_POINTER_MOTION_MASK or
      GDK_POINTER_MOTION_HINT_MASK;
   attributes.visual := gtk_widget_get_visual(widget);
   attributes.colormap := gtk_widget_get_colormap(widget);

   attributes_mask := GDK_WA_X or GDK_WA_Y or GDK_WA_VISUAL or GDK_WA_COLORMAP;
   widget^.window := gdk_window_new(widget^.parent^.window, @attributes,
                                    attributes_mask);

   widget^.style := gtk_style_attach(widget^.style, widget^.window);

   gdk_window_set_user_data(widget^.window, widget);

   gtk_style_set_background(widget^.style, widget^.window, GTK_STATE_ACTIVE);
end;

procedure gtk_dial_size_request (widget : PGtkWidget; 
                                 requisition : PGtkRequisition); cdecl;
begin
   requisition^.width := DIAL_DEFAULT_SIZE;
   requisition^.height := DIAL_DEFAULT_SIZE;
end;

procedure gtk_dial_size_allocate (widget : PGtkWidget;
                                  allocation : PGtkAllocation); cdecl;
var
   dial : PGtkDial; 
begin
   if (widget = nil) or (not GTK_IS_DIAL(widget)) or (allocation = nil) then
      Exit();

   widget^.allocation := allocation^;
   dial := GTK_DIAL(widget);

   if gtk_widget_realized(widget) then
      gdk_window_move_resize(widget^.window,
                             allocation^.x, allocation^.y,
                             allocation^.width, allocation^.height);
   dial^.radius := trunc(min(allocation^.width, allocation^.height) * 0.45);
   dial^.pointer_width := dial^.radius div 5;
end;

function gtk_dial_expose (widget : PGtkWidget; event : PGdkEventExpose) : gboolean; cdecl;
var
   dial : PGtkDial; 
   points : array [0..5] of TGdkPoint;
   s, c : gdouble;
   theta, last, increment : gdouble;
   blankstyle : PGtkStyle;
   xc, yc : gint;
   upper, lower : gint;
   tick_length : gint;
   i, inc : gint;
begin
   if (widget = nil) or (not GTK_IS_DIAL(widget)) or (event = nil) then
      Exit(false);

   if event^.count > 0 then
      Exit(false);
  
   dial := GTK_DIAL(widget);

   xc := widget^.allocation.width div 2;
   yc := widget^.allocation.height div 2;

   upper := trunc(dial^.adjustment^.upper);
   lower := trunc(dial^.adjustment^.lower);

   { Erase old pointer }

   s := sin(dial^.last_angle);
   c := cos(dial^.last_angle);
   dial^.last_angle := dial^.angle;

   points[0].x := trunc(xc + s * dial^.pointer_width / 2);
   points[0].y := trunc(yc + c * dial^.pointer_width / 2);
   points[1].x := trunc(xc + c * dial^.radius);
   points[1].y := trunc(yc - s * dial^.radius);
   points[2].x := trunc(xc - s * dial^.pointer_width / 2);
   points[2].y := trunc(yc - c * dial^.pointer_width / 2);
   points[3].x := trunc(xc - c * dial^.radius / 10);
   points[3].y := trunc(yc + s * dial^.radius / 10);
   points[4].x := points[0].x;
   points[4].y := points[0].y;

   blankstyle := gtk_style_new();
   blankstyle^.bg_gc[GTK_STATE_NORMAL] :=
      widget^.style^.bg_gc[GTK_STATE_NORMAL];
   blankstyle^.dark_gc[GTK_STATE_NORMAL] :=
      widget^.style^.bg_gc[GTK_STATE_NORMAL];
   blankstyle^.light_gc[GTK_STATE_NORMAL] :=
      widget^.style^.bg_gc[GTK_STATE_NORMAL];
   blankstyle^.black_gc :=
      widget^.style^.bg_gc[GTK_STATE_NORMAL];
   blankstyle^.depth :=
      gdk_drawable_get_depth(GDK_DRAWABLE(widget^.window));

   gtk_paint_polygon(blankstyle,
                     widget^.window,
                     GTK_STATE_NORMAL,
                     GTK_SHADOW_OUT,
                     nil,
                     widget,
                     nil,
                     points, 5,
                     false);

   g_object_unref(blankstyle);

   { Draw ticks }

   if (upper - lower) = 0 then
    Exit(false);

   increment := (100 * pi) / (dial^.radius * dial^.radius);

   inc := upper - lower;

   while inc < 100 do inc := 10 * inc;
   while inc >= 1000 do inc := inc div 10;
   last := -1;

   for i := 0 to inc do begin
      theta := (i * pi / (18 * inc / 24.0) - pi / 6.0);

      if (theta - last) < increment then
         continue;     
      last := theta;

      s := sin(theta);
      c := cos(theta);

      if i mod (inc div 10) = 0 then
         tick_length := dial^.pointer_width
      else
         tick_length := dial^.pointer_width div 2;

      gdk_draw_line(widget^.window,
                    widget^.style^.fg_gc[widget^.state],
                    trunc(xc + c * (dial^.radius - tick_length)),
                    trunc(yc - s * (dial^.radius - tick_length)),
                    trunc(xc + c * dial^.radius),
                    trunc(yc - s * dial^.radius));
   end;

   { Draw pointer }

   s := sin(dial^.angle);
   c := cos(dial^.angle);
   dial^.last_angle := dial^.angle;

   points[0].x := xc + trunc(s * dial^.pointer_width / 2);
   points[0].y := yc + trunc(c * dial^.pointer_width / 2);
   points[1].x := xc + trunc(c * dial^.radius);
   points[1].y := yc - trunc(s * dial^.radius);
   points[2].x := xc - trunc(s * dial^.pointer_width / 2);
   points[2].y := yc - trunc(c * dial^.pointer_width / 2);
   points[3].x := xc - trunc(c * dial^.radius / 10);
   points[3].y := yc + trunc(s * dial^.radius / 10);
   points[4].x := points[0].x;
   points[4].y := points[0].y;

   gtk_paint_polygon(widget^.style,
                     widget^.window,
                     GTK_STATE_NORMAL,
                     GTK_SHADOW_OUT,
                     nil,
                     widget,
                     nil,
                     points, 5,
                     true);
   gtk_dial_expose := false;
end;

function gtk_dial_button_press (widget : PGtkWidget;
                                event : PGdkEventButton) : gboolean; cdecl;
var
   dial : PGtkDial; 
   dx, dy : gint;
   s, c : double;
   d_parallel : double;
   d_perpendicular : double;
begin
   if (widget = nil) or (not GTK_IS_DIAL(widget)) or (event = nil) then
      Exit(false);

   dial := GTK_DIAL(widget);

   { Determine if button press was within pointer region - we 
     do this by computing the parallel and perpendicular distance of
     the point where the mouse was pressed from the line passing through
     the pointer }
  
   dx := trunc(event^.x - widget^.allocation.width / 2);
   dy := trunc(widget^.allocation.height div 2 - event^.y);
  
   s := sin(dial^.angle);
   c := cos(dial^.angle);
  
   d_parallel := s * dy + c * dx;
   d_perpendicular := abs(s * dx - c * dy);
  
   if (dial^.button = 0) and
      (d_perpendicular < dial^.pointer_width / 2) and
      (d_parallel > - dial^.pointer_width) then begin
      gtk_grab_add(widget);

      dial^.button := event^.button;

      gtk_dial_update_mouse(dial, trunc(event^.x), trunc(event^.y));
   end;
   gtk_dial_button_press := false;
end;

function gtk_dial_button_release (widget : PGtkWidget;
                                  event : PGdkEventButton): gboolean; cdecl;
var
   dial : PGtkDial; 
begin
   if (widget = nil) or (not GTK_IS_DIAL(widget)) or (event = nil) then
      Exit(false);

   dial := GTK_DIAL(widget);

   if dial^.button = event^.button then begin
      gtk_grab_remove(widget);

      dial^.button := 0;

      if dial^.policy = GTK_UPDATE_DELAYED then
         g_source_remove(dial^.timer);
      
      if (dial^.policy <> GTK_UPDATE_CONTINUOUS) and
         (dial^.old_value <> dial^.adjustment^.value) then
         g_signal_emit_by_name(GTK_OBJECT(dial^.adjustment), 'value_changed');
   end;

   gtk_dial_button_release := false;
end;

function gtk_dial_motion_notify (widget : PGtkWidget;
                                 event : PGdkEventMotion) : gboolean; cdecl;
var
   dial : PGtkDial;
   mods : TGdkModifierType;
   x, y, mask : gint;
begin
   if (widget = nil) or (not GTK_IS_DIAL(widget)) or (event = nil) then
      Exit(false);

   dial := GTK_DIAL(widget);

   if dial^.button <> 0 then begin
      x := trunc(event^.x);
      y := trunc(event^.y);

      if (event^.is_hint = 1) or (event^.window <> widget^.window) then
         gdk_window_get_pointer(widget^.window, @x, @y, @mods);

      case dial^.button of
        1: mask := GDK_BUTTON1_MASK;
        2: mask := GDK_BUTTON2_MASK;
        3: mask := GDK_BUTTON3_MASK;
        else mask := 0;
      end;

      if (mods and mask) <> 0 then
         gtk_dial_update_mouse(dial, x, y);
   end;

   gtk_dial_motion_notify := false;
end;

function gtk_dial_timer (data : gpointer) : gboolean; cdecl;
var
   dial : PGtkDial;
begin
   if (data = nil) or not GTK_IS_DIAL(data) then
      Exit(false);

   dial := GTK_DIAL(data);

   if dial^.policy = GTK_UPDATE_DELAYED then
      g_signal_emit_by_name(GTK_OBJECT(dial^.adjustment), 'value_changed');

   gtk_dial_timer := false;
end;

procedure gtk_dial_update_mouse (dial : PGtkDial; x : gint; y : gint);
var
   xc, yc : gint; 
   old_value : gfloat;
begin
   if (dial = nil) or not GTK_IS_DIAL(dial) then
      Exit();

   xc := GTK_WIDGET(dial)^.allocation.width div 2;
   yc := GTK_WIDGET(dial)^.allocation.height div 2;

   old_value := dial^.adjustment^.value;
   dial^.angle := arctan2(yc - y, x - xc);

   if dial^.angle < -pi / 2.0 then
      dial^.angle := dial^.angle + 2 * pi;

   if dial^.angle < -pi / 6 then
      dial^.angle := -pi / 6;

   if dial^.angle > 7.0 * pi / 6.0 then
      dial^.angle := 7.0 * pi / 6.0;

   dial^.adjustment^.value := dial^.adjustment^.lower +
      (7.0 * pi / 6 - dial^.angle) *
      (dial^.adjustment^.upper - dial^.adjustment^.lower) /
      (4.0 * pi / 3.0);

   if dial^.adjustment^.value <> old_value then begin
      if dial^.policy = GTK_UPDATE_CONTINUOUS then
         g_signal_emit_by_name(GTK_OBJECT (dial^.adjustment), 'value_changed')
      else begin
         gtk_widget_queue_draw(GTK_WIDGET(dial));

         if dial^.policy = GTK_UPDATE_DELAYED then begin
            if dial^.timer <> 0 then
               g_source_remove(dial^.timer);

            dial^.timer := g_timeout_add(SCROLL_DELAY_LENGTH,
                                         @gtk_dial_timer,
                                         gpointer(dial));
         end;
      end;
   end;
end;

procedure gtk_dial_update (dial : PGtkDial);
var
   new_value : gfloat ;
begin
   if (dial = nil) or not GTK_IS_DIAL(dial) then
      Exit();

   new_value := dial^.adjustment^.value;
  
   if new_value < dial^.adjustment^.lower then
      new_value := dial^.adjustment^.lower;

   if new_value > dial^.adjustment^.upper then
      new_value := dial^.adjustment^.upper;

   if new_value <> dial^.adjustment^.value then begin
      dial^.adjustment^.value := new_value;
      g_signal_emit_by_name(GTK_OBJECT(dial^.adjustment), 'value_changed');
   end;

   dial^.angle := 7.0 * pi / 6.0 -
    (new_value - dial^.adjustment^.lower) *
    4.0 * pi / 3.0 / (dial^.adjustment^.upper - dial^.adjustment^.lower);

   gtk_widget_queue_draw(GTK_WIDGET(dial));
end;

procedure gtk_dial_adjustment_changed (adjustment : PGtkAdjustment;
                                       data : gpointer); cdecl;
var
   dial : PGtkDial;
begin
   if (adjustment = nil) or (data = nil) then
      Exit();

   dial := GTK_DIAL(data);

   if (dial^.old_value <> adjustment^.value) or
      (dial^.old_lower <> adjustment^.lower) or
      (dial^.old_upper <> adjustment^.upper) then begin
      gtk_dial_update(dial);
      
      dial^.old_value := adjustment^.value;
      dial^.old_lower := adjustment^.lower;
      dial^.old_upper := adjustment^.upper;
   end;
end;

procedure gtk_dial_adjustment_value_changed (adjustment : PGtkAdjustment;
                                             data : gpointer); cdecl;
var
   dial : PGtkDial;
begin
   if (adjustment = nil) or (data = nil) then
      Exit();
   
   dial := GTK_DIAL(data);

   if dial^.old_value <> adjustment^.value then begin
      gtk_dial_update(dial);

      dial^.old_value := adjustment^.value;
   end;
end;

end.

{*EOF*}
