program helloworld;

uses gtk2, gdk2, glib2;

{ This is a callback function. The data arguments are ignored in this
  example. More on callbacks later }

procedure hello (widget : PGtkWidget; data : gpointer); cdecl;
begin
   writeln('Hello World!');
end;

function delete_event (widget : PGtkWidget; event : PGdkEvent;
                       data : gpointer) : gboolean; cdecl;
  { If you return false from the "delete-event" signal handler, GTK will
    emit the "destroy" signal.  Returning true means you don't want the
    window to be destroyed. This is useful for popping up 'are you sure you
    want to quit?' type dialogs. }

begin
   writeln('delete event occurred');

   { Change true to false and the main window will be destroyed with
     a "delete-event". }
   delete_event := true;
end;

{ Another callback }

procedure destroy (widget : PGtkWidget; data : gpointer); cdecl;
begin
   gtk_main_quit();
end;

var
   window, button : PGtkWidget;

begin
   { This is called in all GTK applications. Arguments are parsed from the
     command line and are returned to the application. }
   gtk_init(@argc, @argv);

   { Create a new window }
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);

   { When the window is given the "delete-event" signal (this is given by
     the window manager, usually by the "close" option, or on the
     titlebar), we ask it to call the delete_event() function as defined
     above. The data passed to the callback function is nil and is ignored
     in the callback function. }
   g_signal_connect(window, 'delete-event',
                    G_CALLBACK(@delete_event), nil);

   { Here we connect the "destroy" event to a signal handler. This event
     occurs when we call gtk_widget_destroy() on the window, or if we
     return false in the "delete-event" callback. }
   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@destroy), nil);

   { Sets the border width of the window. }
   gtk_container_set_border_width(GTK_CONTAINER(window), 10);

   { Creates a new button with the label "Hello World". }
   button := gtk_button_new_with_label('Hello World');

   { When the button receives the "clicked" signal, it will call the
     procedure hello() passing it nil as its argument. }
   g_signal_connect(button, 'clicked',
                    G_CALLBACK(@hello), nil);

   { This will cause the window to be destroyed by calling
     gtk_widget_destroy(window) when "clicked".  Again, the destroy
     signal could come from here, or the window manager. }
   g_signal_connect_swapped(button, 'clicked',
                            G_CALLBACK(@gtk_widget_destroy),
                            window);

   { This packs the button into the window (a gtk container). }
   gtk_container_add(GTK_CONTAINER(window), button);

   { The final step is to display this newly created widget. }
   gtk_widget_show(button);

   { and the window }
   gtk_widget_show(window);

   { All GTK applications must have a gtk_main(). Control ends here and
     waits for an event to occur (like a key press or mouse event). }
   gtk_main();
end.

{*EOF*}

