program ItemFactory;

uses gtk2, gdk2, glib2, sysutils;

{ Obligatory basic callback. }

procedure print_hello (Widget : PGtkWidget; data : gpointer); cdecl;
begin
   writeln('Hello, World!');
end;

{ For the check button }

procedure print_toggle (callback_data : gpointer;
                        callback_action : guint;
                        menu_item : PGtkWidget); cdecl;
begin
    writeln('Check button state - ',
            gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item)));
end;

{ For the radio buttons }

procedure print_selected (callback_data : gpointer;
                          callback_action : guint;
                          menu_item : PGtkWidget); cdecl;
begin
   if gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item)) then
      writeln('Radio button ', callback_action, ' selected');
end;

{ Our menu, an array of GtkItemFactoryEntry structures that defines each
  menu item }

const
   nmenu_items = 16;
   menu_items : array [1..nmenu_items] of TGtkItemFactoryEntry =
   ((path:'/_File'; accelerator:nil; callback:nil; callback_action:0;
     item_type:'<Branch>'),                                    
    (path:'/File/_New'; accelerator:'<ctrl>N';
     callback:TGtkItemFactoryCallback(@print_hello);
     callback_action:0; item_type:nil),
    (path:'/File/_Open'; accelerator:'<ctrl>O';
     callback:TGtkItemFactoryCallback(@print_hello);
     callback_action:0; item_type:nil),
    (path:'/File/_Save'; accelerator:'<ctrl>S';
     callback:TGtkItemFactoryCallback(@print_hello);
     callback_action:0; item_type:nil),
    (path:'/File/Save _As'; accelerator:nil;
     callback:nil; callback_action:0; item_type:'<Item>'),
    (path:'/File/sep1'; accelerator:nil;
     callback:nil; callback_action:0; item_type:'<Separator>'),
    (path:'/File/Quit'; accelerator:'<ctrl>Q';
     callback:TGtkItemFactoryCallback(@gtk_main_quit);
     callback_action:0; item_type:nil),
    (path:'/_Options'; accelerator:nil;
     callback:nil; callback_action:0; item_type:'<Branch>'),
    (path:'/Options/tear'; accelerator:nil; callback:nil; callback_action:0;
     item_type:'<Tearoff>'),
    (path:'/Options/Check'; accelerator:nil;
     callback:TGtkItemFactoryCallback(@print_toggle);
     callback_action:1; item_type:'<CheckItem>'),
    (path:'/Options/sep'; accelerator:nil; callback:nil; callback_action:0;
     item_type:'<Separator>'),
    (path:'/Options/Rad1'; accelerator:nil;
     callback:TGtkItemFactoryCallback(@print_selected);
     callback_action:1; item_type:'<RadioItem>'),
    (path:'/Options/Rad2'; accelerator:nil;
     callback:TGtkItemFactoryCallback(@print_selected);
     callback_action:2; item_type:'/Options/Rad1'),
    (path:'/Options/Rad3'; accelerator:nil;
     callback:TGtkItemFactoryCallback(@print_selected);
     callback_action:3; item_type:'/Options/Rad1'),
    (path:'/_Help'; accelerator:nil;
     callback:nil; callback_action:0; item_type:'<LastBranch>'),
    (path:'/_Help/About'; accelerator:nil;
     callback:nil; callback_action:0; item_type:'<Item>'));

{ Returns a menubar widget made from the above menu }

function get_menubar_menu (window : PGtkWidget) : PGtkWidget;
var
   item_factory : PGtkItemFactory;
   accel_group  : PGtkAccelGroup;
begin
   { Make an accelerator group (shortcut keys) }
   accel_group := gtk_accel_group_new();

   { Make an ItemFactory (that makes a menubar) }

   { This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
   }

   item_factory := gtk_item_factory_new(GTK_TYPE_MENU_BAR, '<main>',
                                        accel_group);

   { This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. }
   gtk_item_factory_create_items(item_factory, nmenu_items, @menu_items, nil);

   { Attach the new accelerator group to the window. }
   gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
   
   { Finally, return the actual menu bar created by the item factory. }
   get_menubar_menu := gtk_item_factory_get_widget(item_factory, '<main>');
end;

{ Popup the menu when the popup button is pressed }

function popup_cb (widget : PGtkWidget; event : PGdkEvent;
                   menu : PGtkWidget) : gboolean; cdecl;
var
   bevent : PGdkEventButton;
begin
   bevent := PGdkEventButton(event);
   { Only take button presses }
   if (event^._type <> GDK_BUTTON_PRESS) then
      popup_cb := false
   else
   begin   
      { Show the menu }
      gtk_menu_popup(GTK_MENU(menu), nil, nil, nil, nil,
                     bevent^.button, bevent^.time);
  
      popup_cb := true;
   end;
end;

{ Same as with get_menubar_menu() but just return a button with a signal to
  call a popup menu }

function get_popup_menu () : PGtkWidget;
var
   item_factory : PGtkItemFactory; 
   button, menu : PGtkWidget;
begin  
   { Same as before but don't bother with the accelerators }
   item_factory := gtk_item_factory_new(GTK_TYPE_MENU, '<main>', nil);
   gtk_item_factory_create_items(item_factory, nmenu_items, @menu_items, nil);
   menu := gtk_item_factory_get_widget(item_factory, '<main>');
  
   { Make a button to activate the popup menu }
   button := gtk_button_new_with_label('Popup');
   { Make the menu popup when clicked }
   g_signal_connect(button, 'event',
                    G_CALLBACK(@popup_cb), gpointer(menu));

   get_popup_menu := button;
end;

{ Same again but return an option menu }

function get_option_menu () : PGtkWidget;
var
   item_factory : PGtkItemFactory; 
   option_menu : PGtkWidget;
begin
   { Same again, not bothering with the accelerators }
   item_factory := gtk_item_factory_new(GTK_TYPE_OPTION_MENU, '<main>',
                                        nil);
   gtk_item_factory_create_items(item_factory, nmenu_items, @menu_items, nil);
   option_menu := gtk_item_factory_get_widget(item_factory, '<main>');

   get_option_menu := option_menu;
end;

{ You have to start somewhere }

var
   window, mainvbox, menubar, option_menu, popup_button : pGtkWidget;

begin
   { Initialize GTK }
   gtk_init(@argc, @argv); 

   { Make a window }
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@gtk_main_quit), nil);
   gtk_window_set_title(GTK_WINDOW(window), 'Item Factory');
   gtk_widget_set_size_request(GTK_WIDGET(window), 300, 200);

   { Make a vbox to put the three menus in }
   mainvbox := gtk_vbox_new(false, 1);
   gtk_container_set_border_width(GTK_CONTAINER(mainvbox), 1);
   gtk_container_add(GTK_CONTAINER(window), mainvbox);
 
   { Get the three types of menu }

   { Note: all three menus are separately created, so they are not the
     same menu }
   menubar := get_menubar_menu(window);
   popup_button := get_popup_menu();
   option_menu := get_option_menu();
   
   { Pack it all together }
   gtk_box_pack_start(GTK_BOX(mainvbox), menubar, false, true, 0);
   gtk_box_pack_end(GTK_BOX(mainvbox), popup_button, false, true, 0);
   gtk_box_pack_end(GTK_BOX(mainvbox), option_menu, false, true, 0);
   
   { Show the widgets }
   gtk_widget_show_all(window);

   { Finished! }
   gtk_main();
end.

{*EOF*}
