program packbox;

uses gtk2, gdk2, glib2, sysutils;

function delete_event (widget : PGtkWidget; event : PGdkEvent;
                       data : gpointer) : gboolean; cdecl;
begin
   gtk_main_quit();
   delete_event := false;
end;

{ Make a new hbox filled with button-labels.  Arguments for the variables
  we're interested are passed in to this function.  We do not show the box,
  but do show everything inside.}

function make_box (homogeneous : boolean; spacing : Longint; expand : boolean;
                   fill : boolean; padding : Longint) : PGtkWidget; cdecl;
var
   box, button : PGtkWidget;
   PadStr      : String;
   PPadStr     : pchar;  //???
begin
   { Create a new hbox with the appropriate homogeneous and spacing settings }
   box := gtk_hbox_new(homogeneous, spacing);
   
   { Create a series of buttons with the appropriate settings }
   button := gtk_button_new_with_label('gtk_box_pack');
   gtk_box_pack_start(GTK_BOX(box), button, expand, fill, padding);
   gtk_widget_show(button);

   button := gtk_button_new_with_label('(box,');
   gtk_box_pack_start(GTK_BOX(box), button, expand, fill, padding);
   gtk_widget_show(button);

   button := gtk_button_new_with_label('button,');
   gtk_box_pack_start(GTK_BOX(box), button, expand, fill, padding);
   gtk_widget_show(button);

   { Create a button with the label depending on the value of expand. }
   if expand = true then
      button := gtk_button_new_with_label('TRUE,')
   else
      button := gtk_button_new_with_label('FALSE,');
   
   gtk_box_pack_start(GTK_BOX(box), button, expand, fill, padding);
   gtk_widget_show(button);

   { This is the same as the button creation for "expand" above,
     but uses the shorthand form (but not in Pascal). }
   if fill then
      button := gtk_button_new_with_label('TRUE')
   else
      button := gtk_button_new_with_label('FALSE');
   gtk_box_pack_start(GTK_BOX(Box), button, expand, fill, padding);
   gtk_widget_show(button);

   PadStr := IntToStr(Padding) + ');';  //!!!
   PPadStr := StrAlloc(Length(PadStr) + 1);
   StrPCopy(PPadStr, PadStr);

   button := gtk_button_new_with_label(PPadStr);
   gtk_box_pack_start(GTK_BOX(box), button, expand, fill, padding);
   gtk_widget_show(button);
   
   make_box := box;
end;

var
   window, button, box1, box2, separator, a_label, quitbox : PGtkWidget; 
   which						   : Integer;

begin
   { Our init, don't forget this! }
   gtk_init(@argc, @argv);

   if argc <> 2 then
   begin
      writeln('usage: packbox num, where num is 1, 2, or 3');
      { This just does cleanup in GTK and exits with an exit status of 1. }
      gtk_exit(1);  //!!! exit
   end;

   which := StrToInt(argv[1]);  //??? strtoint = atoi
   
   { Create our window. }
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   
   { You should always remember to connect the "delete-event" signal
     to the main window.  This is very important for proper intuitive
     behaviour. }
   g_signal_connect(window, 'delete-event',
                    G_CALLBACK(@delete_event), nil);
   gtk_container_set_border_width(GTK_CONTAINER(window),10);

   { We create a vertical box (vbox) to pack the horizontal boxes into.
     This allows us to stack the horizontal boxes filled with buttons
     one on top of the other in this vbox. }
   box1 := gtk_vbox_new(false, 0);
   
   { Which example to show. These correspond to the pictures above. }
   case which of
     1 : begin
            { Create a new label. }
            a_label := gtk_label_new('gtk_hbox_new (FALSE,0);');
            { Align the label to the left side. We'll discuss this function
              and others in the section on Widget Attributes.}
            gtk_misc_set_alignment(GTK_MISC(a_label), 0, 0);
            
            { Pack the label into the vertical box (vbox box1). Remember that
              widgets added to a vbox will be packed one on top of the other
              in order. }
            gtk_box_pack_start(GTK_BOX(box1), a_label, false, false, 0);

            { Show the label.}
            gtk_widget_show(a_label);

            { Call our make box function: homogeneous := false, spacing = 0,
              expand = false, fill := false, padding := 0 }
            box2 := make_box(false, 0, false, false, 0);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);

            { Call our make box function: homogeneous = false, spacing = 0,
              expand = true, fill = false, padding = 0 }
  	    box2 := make_box (false, 0, true, false, 0);
	    gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
	    gtk_widget_show(box2);
            
            { Args are: homogeneous, spacing, expand, fill, padding}
            box2 := make_box (false, 0, true, true, 0);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);
	
            { Creates a separator, we'll learn more about these later, 
              but they are quite simple. }
            separator := gtk_hseparator_new();
	
            { Pack the separator into the vbox. Remember each of these
              widgets is being packed into a vbox, so they'll be stacked
              vertically. }
            gtk_box_pack_start(GTK_BOX(box1), separator, false, true, 5);
            gtk_widget_show(separator);
	
            { Create another new label, and show it. }
            a_label := gtk_label_new('gtk_hbox_new (TRUE, 0);'); 
            gtk_misc_set_alignment(GTK_MISC(a_label), 0, 0); 
            gtk_box_pack_start(GTK_BOX(box1), a_label, false, false, 0);
            gtk_widget_show(a_label);
	
            { Args are: homogeneous, spacing, expand, fill, padding }
            box2 := make_box(true, 0, true, false, 0);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);
	
            { Args are: homogeneous, spacing, expand, fill, padding }
            box2 := make_box(true, 0, true, true, 0);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);
	
            { Another new separator. }
            separator := gtk_hseparator_new();
            { The last 3 arguments to gtk_box_pack_start are:
             expand, fill, padding. }
            gtk_box_pack_start(GTK_BOX(box1), separator, false, true, 5);
            gtk_widget_show(separator);
         end;

     2 : begin
            { Create a new label, remember box1 is a vbox as created 
              near the beginning of main() }
            a_label := gtk_label_new('gtk_hbox_new (false, 10);'); 
            gtk_misc_set_alignment(GTK_MISC(a_label), 0, 0); 
            gtk_box_pack_start(GTK_BOX(box1), a_label, false, false, 0);
            gtk_widget_show(a_label);
	
            { Args are: homogeneous, spacing, expand, fill, padding }
            box2 := make_box(false, 10, true, false, 0);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);
	
            { Args are: homogeneous, spacing, expand, fill, padding }
            box2 := make_box(false, 10, true, true, 0);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);
	
            separator := gtk_hseparator_new();
            { The last 3 arguments to gtk_box_pack_start are:
              expand, fill, padding. }
            gtk_box_pack_start(GTK_BOX(box1), separator, false, true, 5);
            gtk_widget_show(separator);
	
            a_label := gtk_label_new('gtk_hbox_new (false, 0);'); 
            gtk_misc_set_alignment(GTK_MISC(a_label), 0, 0); 
            gtk_box_pack_start(GTK_BOX(box1), a_label, false, false, 0); 
            gtk_widget_show(a_label);                                 
	
            { Args are: homogeneous, spacing, expand, fill, padding }
            box2 := make_box(false, 0, true, false, 10);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);
	
            { Args are: homogeneous, spacing, expand, fill, padding }
            box2 := make_box(false, 0, true, true, 10);
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);
	
            separator := gtk_hseparator_new();
            { The last 3 arguments to gtk_box_pack_start are: expand, fill,
              padding. }
            gtk_box_pack_start(GTK_BOX(box1), separator, false, true, 5);
            gtk_widget_show(separator);
         end;    

     3 : begin
            { This demonstrates the ability to use gtk_box_pack_end() to
              right justify widgets. First, we create a new box as before. }
            box2 := make_box(false, 0, false, false, 0);

            { Create the label that will be put at the end. }
            a_label := gtk_label_new('end'); 
            { Pack it using gtk_box_pack_end(), so it is put on the right
              side of the hbox created in the make_box() call. }
            gtk_box_pack_end(GTK_BOX(box2), a_label, false, false, 0); 
            { Show the label. }                                   
            gtk_widget_show(a_label); 
	
            { Pack box2 into box1 (the vbox remember ? :) }
            gtk_box_pack_start(GTK_BOX(box1), box2, false, false, 0);
            gtk_widget_show(box2);

            { A separator for the bottom. }
            separator := gtk_hseparator_new();
            { This explicitly sets the separator to 400 pixels wide by 5 pixels
              high. This is so the hbox we created will also be 400 pixels wide,
              and the "end" label will be separated from the other labels in the
              hbox. Otherwise, all the widgets in the hbox would be packed as
              close together as possible. }
            gtk_widget_set_size_request(separator, 400, 5);
            { Pack the separator into the vbox (box1) created near the start 
              of main() }
            gtk_box_pack_start(GTK_BOX(box1), separator, false, true, 5);
            gtk_widget_show(separator);
         end;
   end;

   {Create another new hbox... remember we can use as many as we need!}
   quitbox := gtk_hbox_new(false, 0);
   
   { Our quit button }
   button := gtk_button_new_with_label('Quit');
   
   { Setup the signal to terminate the program when the button is clicked }
   g_signal_connect_swapped(button, 'clicked',
			    G_CALLBACK(@gtk_main_quit), window);
   
   { Pack the button into the quitbox. 
     The last 3 arguments to gtk_box_pack_start are:
     expand, fill, padding. }
   gtk_box_pack_start(GTK_BOX(quitbox), button, true, false, 0);
   { Pack the quitbox into the vbox (box1) }
   gtk_box_pack_start(GTK_BOX(box1), quitbox, false, false, 0);

   { Pack the vbox (box1) which now contains all our widgets,
     into the main window }
   gtk_container_add(GTK_CONTAINER(window), box1);

   { And now show everything. }
   gtk_widget_show(button);
   gtk_widget_show(quitbox);
   
   gtk_widget_show(box1);
   { Showing the window last so everything pops up at once. }
   gtk_widget_show(window);

   { And, of course, our main function. }
   gtk_main();
end.

{*EOF*}
