program PanedExample;

uses gtk2, gdk2, glib2, sysutils;

{ Create the list of "messages" }

function create_list() : PGtkWidget;
var
   scrolled_window, tree_view : PGtkWidget;
   model : PGtkListStore;
   iter : TGtkTreeIter;
   cell : PGtkCellRenderer;
   column : PGtkTreeViewColumn;
   i : Integer;
   msg : pgchar;
begin
   { Create a new scrolled window, with scrollbars only if needed }
   scrolled_window := gtk_scrolled_window_new(nil, nil);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);

   model := gtk_list_store_new(1, G_TYPE_STRING);
   tree_view := gtk_tree_view_new();
   gtk_container_add(GTK_CONTAINER(scrolled_window), tree_view);
   gtk_tree_view_set_model(GTK_TREE_VIEW(tree_view), GTK_TREE_MODEL(model));
   gtk_widget_show(tree_view);

   { Add some messages to the window }
   for i := 1 to 10 do
   begin
      msg := g_strdup_printf('Message #%d', i);
      gtk_list_store_append(GTK_LIST_STORE(model), @iter);
      gtk_list_store_set(GTK_LIST_STORE(model), 
                         @iter,
                         0, msg,
                         -1);
   end;
   
   cell := gtk_cell_renderer_text_new();

   column := gtk_tree_view_column_new_with_attributes('Messages',
                                                      cell,
                                                      'text', 0,
                                                      nil);
  
   gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view),
                               GTK_TREE_VIEW_COLUMN(column));

   create_list := scrolled_window;
end;

{ Add some text to our text widget - this is a callback that is invoked
  when our window is realized.  We could also force our window to be
  realized with gtk_widget_realize(), but it would have to be
  part of a hierarchy first }

procedure insert_text (buffer : PGtkTextBuffer);
var
   iter : TGtkTextIter;
begin 
   gtk_text_buffer_get_iter_at_offset(buffer, @iter, 0);

   gtk_text_buffer_insert(buffer, @iter,   
    'From: pathfinder@nasa.gov'#10 +
    'To: mom@nasa.gov'#10 +
    'Subject: Made it!'#10 +
    #10 +
    'We just got in this morning. The weather has been'#10 +
    'great - clear but cold, and there are lots of fun sights.'#10 +
    'Sojourner says hi. See you soon.'#10 +
    ' -Path'+#10, -1);
end;

{ Create a scrolled text area that displays a "message" }

function create_text () : PGtkWidget;
var
   scrolled_window, view : PGtkWidget;
   buffer                : PGtkTextBuffer; 
begin
   view := gtk_text_view_new();
   buffer := gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));

   scrolled_window := gtk_scrolled_window_new(nil, nil);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);

   gtk_container_add(GTK_CONTAINER(scrolled_window), view);
   insert_text(buffer);

   gtk_widget_show_all(scrolled_window);

   create_text := scrolled_window;
end;

var
   window, vpaned, list, text : PGtkWidget;

begin
   gtk_init(@argc, @argv);
   
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(window), 'Paned Windows');
   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@gtk_main_quit), NIL);
   gtk_container_set_border_width(GTK_CONTAINER(window), 10);
   gtk_widget_set_size_request(GTK_WIDGET(window), 450, 400);

   { Create a vpaned widget and add it to our toplevel window }
   vpaned := gtk_vpaned_new();
   gtk_container_add(GTK_CONTAINER(window), vpaned);
   gtk_widget_show(vpaned);

   { Now create the contents of the two halves of the window }
   list := create_list();
   gtk_paned_add1(GTK_PANED(vpaned), list);
   gtk_widget_show(list);

   text := create_text();
   gtk_paned_add2(GTK_PANED(vpaned), text);
   gtk_widget_show(text);
   gtk_widget_show(window);

   gtk_main();
end.

{*EOF*}
