program ProgressBar;

{$mode objfpc}

uses glib2, gtk2, gdk2;

type
   TProgressData  = record
                       window        : PGtkWidget;
                       pbar          : PGtkWidget;
                       timer         : integer;
                       activity_mode : boolean;
                    end;             
   PProgressData = ^TProgressData;

{ Update the value of the progress bar so that we get some movement }

function progress_timeout (data : gpointer) : LongBool; cdecl;
var
   pdata   : PProgressData;
   new_val : gdouble;
begin      
   pdata := PProgressData(data);

   if pdata^.activity_mode then
      gtk_progress_bar_pulse(GTK_PROGRESS_BAR(pdata^.pbar))
   else
   begin
      { Calculate the value of the progress bar using the pointer
        value range set in the adjustment object }
      new_val := gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(pdata^.pbar)) + 0.01;

      if (new_val > 1.0) then
         new_val := 0.0;

      { Set the new value }
      gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pdata^.pbar), new_val);
   end;

   { As this is a timeout function, return TRUE so that
     it continues to get called }
   progress_timeout := true;
end;

{ Callback that toggles the text display within the progress bar trough }

procedure toggle_show_text (widget : PGtkWidget; pdata : PProgressData); cdecl;
var
   text : pgchar;
begin   
   text := gtk_progress_bar_get_text(GTK_PROGRESS_BAR(pdata^.pbar));
   if (text <> nil) and (Length(text) > 0) then
      gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pdata^.pbar), '')
   else
      gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pdata^.pbar), 'some text');
end;

{ Callback that toggles the activity mode of the progress bar }

procedure toggle_activity_mode (widget : PGtkWidget;
                                pdata : PProgressData); cdecl;
begin
   pdata^.activity_mode := not pdata^.activity_mode;
   if pdata^.activity_mode then
      gtk_progress_bar_pulse(GTK_PROGRESS_BAR(pdata^.pbar))
   else
      gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pdata^.pbar), 0.0);
end;

{ Callback that toggles the orientation of the progress bar }

procedure toggle_orientation (widget : PGtkWidget;
                              pdata : PProgressData); cdecl;
begin
   case gtk_progress_bar_get_orientation(GTK_PROGRESS_BAR(pdata^.pbar)) of
     GTK_PROGRESS_LEFT_TO_RIGHT : 
     gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(pdata^.pbar), 
                                      GTK_PROGRESS_RIGHT_TO_LEFT);
     GTK_PROGRESS_RIGHT_TO_LEFT : 
     gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(pdata^.pbar), 
                                      GTK_PROGRESS_LEFT_TO_RIGHT);
     {default : do nothing }
   end;
end;

{ Clean up allocated memory and remove the timer }

procedure destroy_progress (widget : PGtkObject; pdata : PProgressData); cdecl;
begin
   g_source_remove(pdata^.timer);
   pdata^.timer := 0;
   pdata^.window := nil;
   dispose(pdata);
   gtk_main_quit();
end;

var
   pdata : PProgressData;
   align, separator, table, button, check, vbox : PGtkWidget;

begin
   gtk_init(@argc, @argv);

   { Allocate memory for the data that is passed to the callbacks }
   pdata := new(PProgressData);
   pdata^.window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_resizable(GTK_WINDOW(pdata^.window), true);
       
   g_signal_connect(pdata^.window, 'destroy',
                    G_CALLBACK(@destroy_progress), pdata);
   gtk_window_set_title(GTK_WINDOW(pdata^.window), 'GtkProgressBar');
   gtk_container_set_border_width(GTK_CONTAINER(pdata^.window), 0);
       
   vbox := gtk_vbox_new(false, 5);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
   gtk_container_add(GTK_CONTAINER(pdata^.window), vbox);
   gtk_widget_show(vbox);
       
   { Create a centering alignment object }
   align := gtk_alignment_new(0.5, 0.5, 0, 0);
   gtk_box_pack_start(GTK_BOX(vbox), align, false, false, 5);
   gtk_widget_show(align);
       
   { Create the GtkProgressBar }
   pdata^.pbar := gtk_progress_bar_new();
   pdata^.activity_mode := false;
       
   gtk_container_add(GTK_CONTAINER(align), pdata^.pbar);
   gtk_widget_show(pdata^.pbar);

   { Add a timer callback to update the value of the progress bar }
   pdata^.timer := g_timeout_add(100, @progress_timeout, pdata);

   separator := gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(vbox), separator, false, false, 0);
   gtk_widget_show(Separator);

   { rows, columns, homogeneous }
   table := gtk_table_new(2, 3, false);
   gtk_box_pack_start(GTK_BOX(vbox), table, false, true, 0);
   gtk_widget_show(table);
       
   { Add a check button to select displaying of the trough text }
   check := gtk_check_button_new_with_label('Show text');
   gtk_table_attach(GTK_TABLE(table), check, 0, 1, 0, 1,
                    GTK_EXPAND or GTK_FILL, GTK_EXPAND or GTK_FILL, 5, 5);
   g_signal_connect(check, 'clicked',
                    G_CALLBACK(@toggle_show_text), pdata);
   gtk_widget_show(check);

   { Add a check button to toggle activity mode }
   check := gtk_check_button_new_with_label('Activity mode');
   gtk_table_attach(GTK_TABLE(table), check, 0, 1, 1, 2,
                    GTK_EXPAND or GTK_FILL, GTK_EXPAND or GTK_FILL, 5, 5);
   g_signal_connect(check, 'clicked',
                    G_CALLBACK(@toggle_activity_mode), pdata);
   gtk_widget_show(check);
       
   { Add a check button to toggle orientation }
   check := gtk_check_button_new_with_label('Right to Left');
   gtk_table_attach(GTK_TABLE(table), check, 0, 1, 2, 3,
                    GTK_EXPAND or GTK_FILL, GTK_EXPAND or GTK_FILL, 5, 5);
   g_signal_connect(check, 'clicked',
                    G_CALLBACK(@toggle_orientation), pdata);
   gtk_widget_show(check);

   { Add a button to exit the program }
   button := gtk_button_new_with_label('close');
   g_signal_connect_swapped(button, 'clicked',
                            G_CALLBACK(@gtk_widget_destroy),
                            pdata^.window);
   gtk_box_pack_start(GTK_BOX(vbox), button, false, false, 0);

   { This makes it so the button is the default. }
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

   { This grabs this button to be the default button. Simply hitting
     the Enter key will cause this button to activate. }
   gtk_widget_grab_default(button);
   gtk_widget_show(button);
       
   gtk_widget_show(pdata^.window);

   gtk_main();
end.

{*EOF*}
