program ScribbleSimple;

uses glib2, gdk2, gtk2;

{ Backing pixmap for drawing area }

const
   pixmap : PGdkPixmap = nil;

{ Create a new backing pixmap of the appropriate size }

function configure_event (widget : PGtkWidget;
                          event : PGdkEventConfigure): boolean; cdecl;
begin
   if pixmap <> nil then
      g_object_unref(pixmap);

   pixmap := gdk_pixmap_new(widget^.window,
                            widget^.allocation.width,
                            widget^.allocation.height,
                            -1);
   gdk_draw_rectangle(pixmap,
                      gtk_widget_get_style(widget)^.white_gc,
                      gint(true),
                      0, 0,
                      widget^.allocation.width,
                      widget^.allocation.height);

   configure_event := true;
end;

{ Redraw the screen from the backing pixmap }

function expose_event (widget : PGtkWidget;
                       event : PGdkEventExpose) : boolean; cdecl;
begin
   gdk_draw_drawable(widget^.window,
                     gtk_widget_get_style(widget)^.fg_gc[GTK_WIDGET_STATE(widget)],
                     pixmap,
                     event^.area.x, event^.area.y,
                     event^.area.x, event^.area.y,
                     event^.area.width, event^.area.height);
	
   expose_event := false;
end;

{ Draw a rectangle on the screen }

procedure draw_brush (widget : PGtkWidget; x, y : gdouble);
var
   update_rect : TGdkRectangle;
begin
   update_rect.x := trunc(x - 5);
   update_rect.y := trunc(y - 5);
   update_rect.width := 10;
   update_rect.height := 10;
   gdk_draw_rectangle(pixmap,
                      gtk_widget_get_style(widget)^.black_gc,
                      gint(true),
                      update_rect.x, update_rect.y,
                      update_rect.width, update_rect.height);
   gtk_widget_queue_draw_area(widget,
                              update_rect.x, update_rect.y,
                              update_rect.width, update_rect.height);
end;

function button_press_event (widget : PGtkWidget;
                             event: PGdkEventButton) : boolean; cdecl;
begin
   if (event^.button = 1) and (pixmap <> nil) then
      draw_brush(widget, event^.x, event^.y);

   button_press_event := true;
end;

function motion_notify_event (widget : PGtkWidget;
                              event: PGdkEventMotion) : boolean; cdecl;
var
   x, y  : integer;
   state : TGdkModifierType;
begin
   if (event^.is_hint <> 0) then
      gdk_window_get_pointer(event^.window, @x, @y, @state)
   else begin
      x := trunc(event^.x);
      y := trunc(event^.y);
      state := event^.state;
   end;

   if ((state and GDK_BUTTON1_MASK) <> 0) and (pixmap <> nil) then
      draw_brush(widget, x, y);

   motion_notify_event := true;
end;

procedure quit ();
begin
   gtk_main_quit();
end;

var
   window, drawing_area, vbox, button : PGtkWidget;

begin
   gtk_init(@argc, @argv);

   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_name(window, 'Test Input');

   vbox := gtk_vbox_new(false, 0);
   gtk_container_add(GTK_CONTAINER(window), vbox);
   gtk_widget_show(vbox);

   g_signal_connect(window, 'destroy', G_CALLBACK(@quit), nil);

   { Create the drawing area }
   drawing_area := gtk_drawing_area_new ();
   gtk_widget_set_size_request(GTK_WIDGET(drawing_area), 200, 200);
   gtk_box_pack_start(GTK_BOX(vbox), drawing_area, true, true, 0);

   gtk_widget_show(drawing_area);

   { Signals used to handle backing pixmap }
   g_signal_connect(drawing_area, 'expose_event',
                    G_CALLBACK(@expose_event), nil);
   g_signal_connect(drawing_area, 'configure_event',
                    G_CALLBACK(@configure_event), nil);

   { Event signals }
   g_signal_connect(drawing_area, 'motion_notify_event',
                    G_CALLBACK(@motion_notify_event), nil);
   g_signal_connect(drawing_area, 'button_press_event',
                    G_CALLBACK(@button_press_event), nil);

   gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK or
                                       GDK_LEAVE_NOTIFY_MASK or
                                       GDK_BUTTON_PRESS_MASK or
                                       GDK_POINTER_MOTION_MASK or
                                       GDK_POINTER_MOTION_HINT_MASK);

   { .. And a quit button }
   button := gtk_button_new_with_label('Quit');
   gtk_box_pack_start(GTK_BOX(vbox), button, false, false, 0);
   
   g_signal_connect_swapped(button, 'clicked',
                            G_CALLBACK(@gtk_widget_destroy), window);
   gtk_widget_show(button);

   gtk_widget_show(window);
   
   gtk_main();
end.
