program ScribbleXInput;

uses glib2, gdk2, gtk2;

{ Backing pixmap for drawing area }

const
   pixmap : PGdkPixmap = nil;

{ Create a new backing pixmap of the appropriate size }

function configure_event (widget : PGtkWidget;
                          event : PGdkEventConfigure): boolean; cdecl;
begin
   if pixmap <> nil then
      g_object_unref(pixmap);

   pixmap := gdk_pixmap_new(widget^.window,
                            widget^.allocation.width,
                            widget^.allocation.height,
                            -1);
   gdk_draw_rectangle(pixmap,
                      gtk_widget_get_style(widget)^.white_gc,
                      gint(true),
                      0, 0,
                      widget^.allocation.width,
                      widget^.allocation.height);

   configure_event := true;
end;

{ Redraw the screen from the backing pixmap }

function expose_event (widget : PGtkWidget;
                       event : PGdkEventExpose) : boolean; cdecl;
begin
   gdk_draw_drawable(widget^.window,
                     gtk_widget_get_style(widget)^.fg_gc[GTK_WIDGET_STATE(widget)],
                     pixmap,
                     event^.area.x, event^.area.y,
                     event^.area.x, event^.area.y,
                     event^.area.width, event^.area.height);
	
   expose_event := false;
end;

{ Draw a rectangle on the screen, size depending on pressure,
  and color on the type of device }

procedure draw_brush (widget : PGtkWidget; source : TGdkInputSource;
                      x, y : gdouble;
                      pressure : gdouble);
var
   gc : PGdkGC;
   update_rect : TGdkRectangle;
begin
   case source of
     GDK_SOURCE_MOUSE : 
     gc := gtk_widget_get_style(widget)^.dark_gc[GTK_WIDGET_STATE(widget)];
     GDK_SOURCE_PEN : 
     gc := gtk_widget_get_style(widget)^.black_gc;
     GDK_SOURCE_ERASER : 
     gc := gtk_widget_get_style(widget)^.white_gc;
     else 
     gc := gtk_widget_get_style(widget)^.light_gc[GTK_WIDGET_STATE(widget)];
   end;

   update_rect.x := trunc(x - 10 * pressure);
   update_rect.y := trunc(y - 10 * pressure);
      writeln('i tutaj');  //tmp
   update_rect.width := trunc(20 * pressure);
   update_rect.height := trunc(20 * pressure);
   gdk_draw_rectangle(pixmap, gc, gint(true),
                      update_rect.x, update_rect.y,
                      update_rect.width, update_rect.height);
   gtk_widget_queue_draw_area(widget, 
                              update_rect.x, update_rect.y,
                              update_rect.width, update_rect.height);
end;

procedure print_button_press (device : PGdkDevice);
begin
   g_print('Button press on device "%s"'#10, device^.name);
end;

function button_press_event (widget : PGtkWidget;
                             event: PGdkEventButton) : boolean; cdecl;
var
   pressure : gdouble;
begin
   print_button_press(event^.device);
  
   if (event^.button = 1) and (pixmap <> nil) then begin
      gdk_event_get_axis(PGdkEvent(event), GDK_AXIS_PRESSURE, @pressure);
      draw_brush(widget, event^.device^.source, event^.x, event^.y, pressure);
   end;

   button_press_event := true;
end;

function motion_notify_event (widget : PGtkWidget;
                              event: PGdkEventMotion) : boolean; cdecl;
var
   x, y     : gdouble;
   pressure : gdouble;
   state    : TGdkModifierType;
begin
   if (event^.is_hint <> 0) then begin
      gdk_device_get_state(event^.device, event^.window, nil, @state);
      gdk_event_get_axis(PGdkEvent(event), GDK_AXIS_X, @x);
      gdk_event_get_axis(PGdkEvent(event), GDK_AXIS_Y, @y);
      gdk_event_get_axis(PGdkEvent(event), GDK_AXIS_PRESSURE, @pressure);
   end
   else begin
      x := trunc(event^.x);
      y := trunc(event^.y);
      gdk_event_get_axis(PGdkEvent(event), GDK_AXIS_PRESSURE, @pressure);
      state := event^.state;
   end;
    
   if ((state and GDK_BUTTON1_MASK) <> 0) and (pixmap <> nil) then
      draw_brush(widget, event^.device^.source, x, y, pressure);
  
   motion_notify_event := true;
end;

var
   inputd : PGtkWidget = nil;

procedure input_dialog_destroy (w : PGtkWidget; data :gpointer); cdecl;
begin
   inputd := nil; 
end;

procedure create_input_dialog ();
begin
   if (inputd= nil) then begin
      inputd := gtk_input_dialog_new();

      g_signal_connect(inputd, 'destroy',
                       G_CALLBACK(@input_dialog_destroy), inputd);
      g_signal_connect_swapped(GTK_INPUT_DIALOG(inputd)^.close_button,
                               'clicked',
                               G_CALLBACK(@gtk_widget_hide),
                               inputd);
      gtk_widget_hide(GTK_INPUT_DIALOG(inputd)^.save_button);

      gtk_widget_show(inputd);
   end
   else begin
      if not GTK_WIDGET_MAPPED(inputd) then
         gtk_widget_show(inputd)
      else
         gdk_window_raise(inputd^.window);
   end;
end;

var
   window, drawing_area, vbox, button : PGtkWidget;

begin
   gtk_init(@argc, @argv);

   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_name(window, 'Test Input');

   vbox := gtk_vbox_new(false, 0);
   gtk_container_add(GTK_CONTAINER(window), vbox);
   gtk_widget_show(vbox);

   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@gtk_main_quit), nil);

   { Create the drawing area }
   drawing_area := gtk_drawing_area_new ();
   gtk_widget_set_size_request(GTK_WIDGET(drawing_area), 200, 200);
   gtk_box_pack_start(GTK_BOX(vbox), drawing_area, true, true, 0);

   gtk_widget_show(drawing_area);

   { Signals used to handle backing pixmap }
   g_signal_connect(drawing_area, 'expose-event',
                    G_CALLBACK(@expose_event), nil);
   g_signal_connect(drawing_area, 'configure-event',
                    G_CALLBACK(@configure_event), nil);

   { Event signals }
   g_signal_connect(drawing_area, 'motion-notify-event',
                    G_CALLBACK(@motion_notify_event), nil);
   g_signal_connect(drawing_area, 'button-press-event',
                    G_CALLBACK(@button_press_event), nil);

   gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK or
                                       GDK_LEAVE_NOTIFY_MASK or
                                       GDK_BUTTON_PRESS_MASK or
                                       GDK_POINTER_MOTION_MASK or
                                       GDK_POINTER_MOTION_HINT_MASK);

   { The following call enables tracking and processing of extension
     events for the drawing area }
   gtk_widget_set_extension_events(drawing_area, GDK_EXTENSION_EVENTS_CURSOR);

   { .. And some buttons }
   button := gtk_button_new_with_label('Input Dialog');
   gtk_box_pack_start(GTK_BOX(vbox), button, false, false, 0);

   g_signal_connect(button, 'clicked',
                    G_CALLBACK(@create_input_dialog), nil);
   gtk_widget_show(button);

   button := gtk_button_new_with_label('Quit');
   gtk_box_pack_start(GTK_BOX(vbox), button, false, false, 0);

   g_signal_connect_swapped(button, 'clicked',
                            G_CALLBACK(@gtk_widget_destroy), window);
   gtk_widget_show(button);

   gtk_widget_show(window);

   gtk_main();
end.

{*EOF*}
