program SetSelection;

uses gtk2, gdk2, glib2, sysutils;

var
   selection_button, selection_widget : PGtkWidget;

{ Callback when the user toggles the selection }

procedure selection_toggled (widget : PGtkWidget;
                             var have_selection : gboolean); cdecl;
begin
   if active(GTK_TOGGLE_BUTTON(widget)^) <> 0 then begin
      have_selection := gtk_selection_owner_set(widget,
                                                GDK_SELECTION_PRIMARY,
                                                GDK_CURRENT_TIME);
      { If claiming the selection failed, we return the button
        to the out state }
      if not have_selection then
	 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), false);
   end
   else
      if have_selection then begin
	 { Before clearing the selection by setting the owner to nil,
	   we check if we are the actual owner }
	 if gdk_selection_owner_get(GDK_SELECTION_PRIMARY) = widget^.window then
	    gtk_selection_owner_set(nil, GDK_SELECTION_PRIMARY,
                                    GDK_CURRENT_TIME);
	 have_selection := false;
      end;
end;

{ Called when another application claims the selection }

function selection_clear (widget : PGtkWidget;
                          event : PGdkEventSelection;
                          var have_selection : gboolean) : gboolean; cdecl;
begin
   have_selection := false;
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), false);
   selection_clear := true;
end;

{ Supplies the current time as the selection. }

procedure selection_handle (widget         : PGtkWidget;
                            selection_data : PGtkSelectionData;
                            info           : guint;
                            time_stamp     : guint;
                            data           : gpointer); cdecl;
var
   timestr : pgchar;
   t	   : tDateTime; { from sysutils }
begin
   { time_t is a C structure, the rest of this callback would need to be translated
   appropriately if GdkAtom is ever implemented in Pascal /////// }
   { time_t current_time; }
   { current_time = time(NULL); }
   { timestr = asctime (localtime(&current_time)); }
   t := now;
   timestr := pgchar(TimeToStr(t));
   { When we return a single string, it should not be null terminated.
     That will be done for us }
   gtk_selection_data_set(selection_data, GDK_SELECTION_TYPE_STRING,
                          8, pguchar(timestr), strlen(timestr));
end;

var
   window : PGtkWidget;
   have_selection : boolean = false;

begin
   gtk_init(@argc, @argv);
   
   { Create the toplevel window }
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(window), 'Event Box');
   gtk_container_set_border_width(GTK_CONTAINER (window), 10);
   
   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@gtk_main_quit), nil);

   { Create a toggle button to act as the selection }
   selection_widget := gtk_invisible_new();
   selection_button := gtk_toggle_button_new_with_label('Claim Selection');
   gtk_container_add(GTK_CONTAINER(window), selection_button);
   gtk_widget_show(selection_button);

   g_signal_connect(selection_button, 'toggled',
                    G_CALLBACK(@selection_toggled), @have_selection);
   g_signal_connect(selection_widget, 'selection_clear_event',
                    G_CALLBACK(@selection_clear), @have_selection);

   gtk_selection_add_target(selection_widget,
                            GDK_SELECTION_PRIMARY,
                            GDK_SELECTION_TYPE_STRING,
                            1);
   g_signal_connect(selection_widget, 'selection_get',
                    G_CALLBACK(@selection_handle), @have_selection);

   gtk_widget_show(selection_button);
   gtk_widget_show(window);
   
   gtk_main();
end.

{*EOF*}