program TextExample;

uses gtk2, gdk2, glib2, sysutils, classes;

procedure text_toggle_editable (checkbutton : PGtkWidget;
                                text : PGtkWidget); cdecl;
begin
   gtk_text_set_editable(GTK_TEXT(text),
                         gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton)));
end;

procedure text_toggle_word_wrap (checkbutton : PGtkWidget;
                                 text : PGtkWidget); cdecl;
begin
   gtk_text_set_word_wrap(GTK_TEXT(text),
                          gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton)));
end;

procedure close_application (widget : PGtkWidget; data : gpointer); cdecl;
begin
   gtk_main_quit();
end;

var
   window, box1, box2, hbox, button, check, separator : PGtkWidget;
   table, vscrollbar, text			      : PGtkWidget;
   cmap						      : PGdkColormap;
   color					      : PGdkColor;
   fixed_font					      : PGdkFont;
   s						      : String;
   t						      : TStringList;
   i, j						      : Longint;
   s_as_char_ptr				      : pchar;
   curr_length					      : Integer;

   {infile : FILE;}

begin
   gtk_init(@argc, @argv);
   
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_size_request(GTK_WIDGET(window), 600, 500);
   gtk_window_set_resizable(GTK_WINDOW(window), true);
   g_signal_connect(window, 'destroy',
                    G_CALLBACK(@close_application), NIL);
   gtk_window_set_title(GTK_WINDOW(window), 'Text Widget Example');
   gtk_container_set_border_width(GTK_CONTAINER(window), 0);

   box1 := gtk_vbox_new(false, 0);
   gtk_container_add(GTK_CONTAINER(window), box1);
   gtk_widget_show(box1);

   box2 := gtk_vbox_new(false, 10);
   gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
   gtk_box_pack_start(GTK_BOX(box1), box2, true, true, 0);
   gtk_widget_show(box2);

   table := gtk_table_new(2, 2, false);
   gtk_table_set_row_spacing(GTK_TABLE(table), 0, 2);
   gtk_table_set_col_spacing(GTK_TABLE(table), 0, 2);
   gtk_box_pack_start(GTK_BOX(box2), table, true, true, 0);
   gtk_widget_show(table);

   { Create the GtkText widget }
   text := gtk_text_new(nil, nil);
   gtk_text_set_editable(GTK_TEXT(text), true);
   gtk_table_attach(GTK_TABLE(table), text, 0, 1, 0, 1,
                    GTK_EXPAND or GTK_SHRINK or GTK_FILL,
		    GTK_EXPAND or GTK_SHRINK or GTK_FILL, 0, 0);
   gtk_widget_show(text);

   { Add a vertical scrollbar to the GtkText widget }
   vscrollbar := gtk_vscrollbar_new(GTK_TEXT(text)^.vadj);
   gtk_table_attach(GTK_TABLE(table), vscrollbar, 1, 2, 0, 1, GTK_FILL,
		    GTK_EXPAND or GTK_SHRINK or GTK_FILL, 0, 0);
   gtk_widget_show(vscrollbar);
   
   { Get the system color map and allocate the color red }
   cmap := gdk_colormap_get_system();
   color := new(PGdkColor);  //!!!???
   with color^ do
      begin
	 red := 65535; { 0xffff }
	 green := 0;
	 blue := 0;
      end;
   if gdk_colormap_alloc_color(cmap, color, false, true) then
      writeln('couldn''t allocate color');

   { Load a fixed font }
   fixed_font := gdk_font_load('-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*');
   
   { Realizing a widget creates a window for it,
     ready for us to insert some text }
   gtk_widget_realize(GTK_WIDGET(text));
   
   { Freeze the text widget, ready for multiple updates }
   gtk_text_freeze(GTK_TEXT(text));

   { Insert some colored text }
   gtk_text_insert(GTK_TEXT(text), nil, @text^.style^.black, nil,
                   'Supports ', -1);
   gtk_text_insert(GTK_TEXT(text), nil, color, nil, 'colored ', -1);
   gtk_text_insert(GTK_TEXT(text), nil, @text^.style^.black, nil,
                   'text and different ', -1);
   gtk_text_insert(GTK_TEXT(text), fixed_font, @text^.style^.black, nil,
                   'fonts'+#10+#10, -1);

   { Load the file text.pas into the text window }
   t := tStringList.create;  //!!! calosc
   t.LoadFromFile('text.pas');
   j := t.Count - 1;
   for i := 0 to j do begin
      s := t.Strings[i] + #10;
      curr_length := length(s);
      s_as_char_ptr := StrAlloc(curr_length + 1);
      StrPCopy(s_as_char_ptr, s);
      gtk_text_insert(GTK_TEXT(text), fixed_font, nil, nil, s_as_char_ptr, curr_length);
   end;
   t.Clear;
   t.Free;

   { Thaw the text widget, allowing the updates to become visible }
   gtk_text_thaw(GTK_TEXT(text));

   hbox := gtk_hbutton_box_new();
   gtk_box_pack_start(GTK_BOX(box2), hbox, false, false, 0);
   gtk_widget_show(hbox);

   check := gtk_check_button_new_with_label('Editable');
   gtk_box_pack_start(GTK_BOX(hbox), check, false, false, 0);
   g_signal_connect(check, 'toggled',
                    G_CALLBACK(@text_toggle_editable), GTK_TEXT(text));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), true);
   gtk_widget_show(check);
   check := gtk_check_button_new_with_label('Wrap Words');
   gtk_box_pack_start(GTK_BOX(hbox), check, false, true, 0);
   g_signal_connect(check, 'toggled',
                    G_CALLBACK(@text_toggle_word_wrap), GTK_TEXT(text));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), false);
   gtk_widget_show(check);

   separator := gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(box1), separator, false, true, 0);
   gtk_widget_show(separator);

   box2 := gtk_vbox_new(false, 10);
   gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
   gtk_box_pack_start(GTK_BOX (box1), box2, false, true, 0);
   gtk_widget_show(box2);

   button := gtk_button_new_with_label('close');
   g_signal_connect(button, 'clicked',
                    G_CALLBACK(@close_application), nil);
   gtk_box_pack_start(GTK_BOX(box2), button, true, true, 0);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default(button);
   gtk_widget_show(button);

   gtk_widget_show(window);

   gtk_main();
end.

{*EOF*}
