program ToolbarExample;

uses glib2, gdk2, gtk2;

{ This function is connected to the Close button or closing the window
  from the WM }

function delete_event (widget : PGtkWidget; event : PGdkEvent;
                       data : gpointer) : boolean; cdecl;
begin
   gtk_main_quit();
   delete_event := false;
end;

var
   close_button : PGtkToolItem;  { This button will emit signal to close
                                   application }
   tooltips_button,  { to enable/disable tooltips }
   text_button : PGtkToolItem;
   icon_button : PGtkToolItem;
   both_button : PGtkToolItem;  { radio buttons for toolbar style }
   entry : PGtkWidget;  { a text entry to show packing any widget into
                          toolbar }

{ That's easy... when one of the buttons is toggled, we just check
  which one is active and set the style of the toolbar accordingly.
  ATTENTION: our toolbar is passed as data to callback! }

procedure radio_event (widget : PGtkWidget; data : gpointer); cdecl;
begin
   if gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(text_button)) then
      gtk_toolbar_set_style(GTK_TOOLBAR(data), GTK_TOOLBAR_TEXT)
   else if gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(icon_button)) then
      gtk_toolbar_set_style(GTK_TOOLBAR(data), GTK_TOOLBAR_ICONS)
   else if gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(both_button)) then
      gtk_toolbar_set_style(GTK_TOOLBAR(data), GTK_TOOLBAR_BOTH);
end;

{ Even easier, just check given toggle button and enable/disable tooltips}

procedure toggle_event (widget : PGtkWidget; data : gpointer); cdecl;
begin
   g_object_set(GTK_SETTINGS(data), 'gtk-enable-tooltips',
                gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(widget)),
                nil);
end;

var
   { Here is our main window (a dialog) and a handle for the handlebox } 
   dialog, handlebox : PGtkWidget;

   { OK, we need a toolbar, and an icon widget to put the icon in
     (but we'll create a separate widget for each button) }
   toolbar, iconw : PGtkWidget;
   gap, item : PGtkToolItem;

   default_settings : PGtkSettings;

begin
   { This is called in all GTK application. }
   gtk_init(@argc, @argv);

   default_settings := gtk_settings_get_default();
   g_object_set(default_settings, 'gtk-button-images', true, nil);
   
   { Create a new window with a given title, and nice size }
   dialog := gtk_dialog_new();
   gtk_window_set_title(GTK_WINDOW(dialog) , 'GTKToolbar Tutorial');
   gtk_widget_set_size_request(GTK_WIDGET(dialog), 600 , 300);
   g_object_set(GTK_WINDOW(dialog), 'allow-shrink', true, nil);

   { Typically we quit if someone tries to close us }
   g_signal_connect(dialog, 'delete-event',
                    G_CALLBACK(@delete_event), nil);

   { We need to realize the window because we use pixmaps for items
     on the toolbar }
   gtk_widget_realize(dialog);
   
   { To make it nice we'll put the toolbar into the handle box,
     so that it can be detached from the main window }
   handlebox := gtk_handle_box_new();
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)^.vbox),
                      handlebox, false, false, 5);

   { Toolbar will be horizontal, with both icons and text,
     and with 5pxl spaces between items and finally, we'll also
     put it into our handlebox }
   toolbar := gtk_toolbar_new();
   gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar),
                               GTK_ORIENTATION_HORIZONTAL);
   gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_BOTH);
   gtk_container_set_border_width(GTK_CONTAINER(toolbar), 5);
   gtk_container_add(GTK_CONTAINER(handlebox), toolbar);

   { Our first item is close button }
   iconw := gtk_image_new_from_file('gtk.xpm'); { icon widget }
   gtk_widget_show(iconw);
   close_button := gtk_tool_button_new(iconw,     { icon widget }
                                       'Close');  { button label }
   gtk_widget_set(GTK_WIDGET(close_button), 'tooltip-text',
                  'Closes this app', nil);  { this button's tooltip }
   g_signal_connect(close_button, 'clicked', G_CALLBACK(@delete_event), nil);
   gtk_widget_show(GTK_WIDGET(close_button));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), close_button, -1);
   { Space after item }
   gap := gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gap, -1);
   gtk_widget_show(GTK_WIDGET(gap));

   { Now, let's make our radio buttons group... }
   iconw := gtk_image_new_from_file('gtk.xpm'); { icon widget }
   gtk_widget_show(iconw);
   icon_button := gtk_radio_tool_button_new(NULL);
   gtk_tool_button_set_label(GTK_TOOL_BUTTON(icon_button), 'Icon');
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(icon_button), iconw);
   gtk_widget_set(GTK_WIDGET(icon_button), 'tooltip-text',
                  'Only icons in toolbar', nil);
   g_signal_connect(icon_button, 'clicked', G_CALLBACK(@radio_event), toolbar);
   gtk_widget_show(GTK_WIDGET(icon_button));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), icon_button, -1);
   gap := gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gap, -1);
   gtk_widget_show(GTK_WIDGET(gap));

   { Following radio buttons refer to previous ones }
   iconw := gtk_image_new_from_file('gtk.xpm'); { icon widget }
   gtk_widget_show(iconw);
   text_button := gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(icon_button));
   gtk_tool_button_set_label(GTK_TOOL_BUTTON(text_button), 'Text');
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(text_button), iconw);
   gtk_widget_set(GTK_WIDGET(text_button), 'tooltip-text',
                  'Only texts in toolbar', nil);
   g_signal_connect(text_button, 'clicked', G_CALLBACK(@radio_event), toolbar);
   gtk_widget_show(GTK_WIDGET(text_button));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), text_button, -1);
   gap := gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gap, -1);
   gtk_widget_show(GTK_WIDGET(gap));

   iconw := gtk_image_new_from_file('gtk.xpm'); { icon widget }
   gtk_widget_show(iconw);
   both_button := gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(icon_button));
   gtk_tool_button_set_label(GTK_TOOL_BUTTON(both_button), 'Both');
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(both_button), iconw);
   gtk_widget_set(GTK_WIDGET(both_button), 'tooltip-text',
                  'Icons and text in toolbar', nil);
   g_signal_connect(both_button, 'clicked', G_CALLBACK(@radio_event), toolbar);
   gtk_widget_show(GTK_WIDGET(both_button));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), both_button, -1);
   gap := gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gap, -1);
   gtk_widget_show(GTK_WIDGET(gap));
   gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(both_button), TRUE);

   { Here we have just a simple toggle button }
   iconw := gtk_image_new_from_file('gtk.xpm'); { icon widget }
   gtk_widget_show(iconw);
   tooltips_button := gtk_toggle_tool_button_new();
   gtk_tool_button_set_label(GTK_TOOL_BUTTON(tooltips_button), 'Tooltips');
   gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(tooltips_button), iconw);
   gtk_widget_set(GTK_WIDGET(tooltips_button), 'tooltip-text',
                  'Toolbar with or without tips', nil);
   g_signal_connect(tooltips_button, 'clicked',
                    G_CALLBACK(@toggle_event), default_settings);
   gtk_widget_show(GTK_WIDGET(tooltips_button));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tooltips_button, -1);
   gap := gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gap, -1);
   gtk_widget_show(GTK_WIDGET(gap));
   gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(tooltips_button),
                                     true);

   { To pack a widget into toolbar, we only have to create it
     and append it with an appropriate tooltip }
   entry := gtk_entry_new();
   item := gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(item), entry);
   gtk_widget_set(entry,'tooltip-text', 'This is just an entry', nil);
   { Well, it isn't created within the toolbar, so we must still show it }
   gtk_widget_show(entry);
   gtk_widget_show(GTK_WIDGET(item));
   gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);

   { That's it ! let's show everything. }
   gtk_widget_show(toolbar);
   gtk_widget_show(handlebox);
   gtk_widget_show(dialog);

   { Rest in gtk_main and wait for the fun to begin! }
   gtk_main();
end.

{*EOF*}
