program TreeExample;

uses gtk2, glib2, gdk2;

{ For all the GtkItem:: and GtkTreeItem:: signals }

procedure cb_itemsignal (item : PGtkWidget; signame : pchar); cdecl;
var
   name	: pchar;
   a_label : PGtkLabel;
   level : Integer;	  
begin
   { It's a Bin, so it has one child, which we know to be a label, so get that }
   a_label := GTK_LABEL(GTK_BIN(item)^.child);
   { Get the text of the label }
   name := gtk_label_get_text(a_label);
   { Get the level of the tree which the item is in }
   level := GTK_TREE(item^.parent)^.level;
   g_print('%s called for item %s->%p, level %d'#10, signame, name, item, level);
end;

{ Note that this is never called }

procedure cb_unselect_child (root_tree : PGtkWidget; child : PGtkWidget;
                             subtree : PGtkWidget); cdecl;
begin
   g_print('unselect_child called for root tree %p, subtree %p, child %p'#10,
           root_tree, subtree, child);
end;

{ Note that this is called every time the user clicks on an item,
  whether it is already selected or not }

procedure cb_select_child (root_tree : PGtkWidget; child : PGtkWidget;
                           subtree : PGtkWidget); cdecl;
begin
   g_print('select_child called for root tree %p, subtree %p, child %p'#10,
           root_tree, subtree, child);
end;

procedure cb_selection_changed (tree : PGtkWidget); cdecl;
var
   i	   : pGList;
   name	   : pchar;
   a_label : PGtkLabel;	
   item	   : PGtkWidget;
begin
   g_print('selection_change called for tree %p'#10, tree);
   writeln('selected objects are:');
   i := GTK_TREE_SELECTION_OLD(GTK_TREE(tree));
   while (i <> nil) do begin
      { Get a GtkWidget pointer from the list node }
      item := GTK_WIDGET(i^.data);
      a_label := GTK_LABEL(GTK_BIN(item)^.child);
      name := gtk_label_get_text(a_label);
      g_print('\t%s on level %d'#10, name, GTK_TREE(item^.parent)^.level);
      i := i^.next;
   end;
end;

const
   itemnames : array [0..4] of pgchar =
   ('Foo', 'Bar', 'Baz', 'Quux', 'Maurice');   

var
   window, scrolled_win, tree, subtree, item, subitem : PGtkWidget;
   i, j : Integer;

begin
   gtk_init(@argc, @argv);

   { A generic toplevel window }
   window := gtk_window_new(GTK_WINDOW_TOPLEVEL);
   g_signal_connect(G_OBJECT(window), 'delete_event',
                    G_CALLBACK(@gtk_main_quit), nil);
   gtk_container_set_border_width(GTK_CONTAINER(window), 5);

   { A generic scrolled window }
   scrolled_win := gtk_scrolled_window_new(nil, nil);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
                                  GTK_POLICY_AUTOMATIC,
				  GTK_POLICY_AUTOMATIC);
   gtk_widget_set_size_request(scrolled_win, 150, 200);
   gtk_container_add(GTK_CONTAINER(window), scrolled_win);
   gtk_widget_show(scrolled_win);

   { Create the root tree }
   tree := gtk_tree_new();
   g_print('root tree is %p'#10, tree);
   { Connect all GtkTree:: signals }
   g_signal_connect(G_OBJECT(tree), 'select_child',
                    G_CALLBACK(@cb_select_child), @tree);
   g_signal_connect(G_OBJECT(tree), 'unselect_child',
                    G_CALLBACK(@cb_unselect_child), @tree);
   g_signal_connect(G_OBJECT(tree), 'selection_changed',
                    G_CALLBACK(@cb_selection_changed), @tree);
   { Add it to the scrolled window }
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_win),
                                         tree);
   { Set the selection mode }
   gtk_tree_set_selection_mode(GTK_TREE(tree), GTK_SELECTION_MULTIPLE);
   { Show it }
   gtk_widget_show(tree);

   for i := 0 to 4 do begin
      { Create a tree item }
      item := gtk_tree_item_new_with_label(itemnames[i]);
      { Connect all GtkItem:: and GtkTreeItem:: signals }
      g_signal_connect(G_OBJECT(item), 'select',
                       G_CALLBACK(@cb_itemsignal), pchar('select'));
      g_signal_connect(G_OBJECT(item), 'deselect',
                       G_CALLBACK(@cb_itemsignal), pchar('deselect'));
      g_signal_connect(G_OBJECT(item), 'toggle',
                       G_CALLBACK(@cb_itemsignal), pchar('toggle'));
      g_signal_connect(G_OBJECT(item), 'expand',
                       G_CALLBACK(@cb_itemsignal), pchar('expand'));
      g_signal_connect(G_OBJECT(item), 'collapse',
                       G_CALLBACK(@cb_itemsignal), pchar('collapse'));
      { Add it to the parent tree }
      gtk_tree_append(GTK_TREE(tree), item);
      { Show it - this can be done at any time}
      gtk_widget_show(item);
      { Create this item's subtree }
      subtree := gtk_tree_new();
      g_print('-> item %s->%p, subtree %p'#10, itemnames[i], item,
              subtree);

      { This is still necessary if you want these signals to be called
        for the subtree's children. Note that selection_change will be
        signalled for the root tree regardless.}
      g_signal_connect(G_OBJECT(subtree), 'select_child',
                       G_CALLBACK(@cb_select_child), @subtree);
      g_signal_connect(G_OBJECT(subtree), 'unselect_child',
                       G_CALLBACK(@cb_unselect_child), @subtree);
      { This has absolutely no effect, because it is completely ignored
        in subtrees }
      gtk_tree_set_selection_mode(GTK_TREE(subtree), GTK_SELECTION_SINGLE);
      { Neither does this, but for a rather different reason - the
        view_mode and view_line values of a tree are propagated
        to subtrees when they are mapped. So, setting it later on
        would actually have a (somewhat unpredictable) effect}
      gtk_tree_set_view_mode(GTK_TREE(subtree), GTK_TREE_VIEW_ITEM);
      { Set this item's subtree - note that you cannot do this until
        AFTER the item has been added to its parent tree!}
      gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);
      
      for j := 0 to 4 do begin
	 { Create a subtree item, in much the same way }
	 subitem := gtk_tree_item_new_with_label(itemnames[j]);
	 { Connect all GtkItem:: and GtkTreeItem:: signals}
	 g_signal_connect(G_OBJECT(subitem), 'select',
                          G_CALLBACK(@cb_itemsignal), pchar('select'));
	 g_signal_connect(G_OBJECT(subitem), 'deselect',
                          G_CALLBACK(@cb_itemsignal), pchar('deselect'));
	 g_signal_connect(G_OBJECT(subitem), 'toggle',
                          G_CALLBACK(@cb_itemsignal), pchar('toggle'));
	 g_signal_connect(G_OBJECT(subitem), 'expand',
                          G_CALLBACK(@cb_itemsignal), pchar('expand'));
	 g_signal_connect(G_OBJECT(subitem), 'collapse',
                          G_CALLBACK(@cb_itemsignal), pchar('collapse'));
	 g_print('-> -> item %s->%p'#10, itemnames[j], subitem);
	 { Add it to its parent tree }
	 gtk_tree_append(GTK_TREE(subtree), subitem);
	 { Show it }
	 gtk_widget_show(subitem);
      end;
   end;

   { Show the window and loop endlessly }
   gtk_widget_show(window);
   gtk_main();
end.

{*EOF*}
